'''Temporarily store a blob in Launchpad and get a ticket for it.

Copyright (C) 2007 Canonical Ltd.
Author: Martin Pitt <martin.pitt@ubuntu.com>

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
the full text of the license.
'''

import urllib2, time, httplib

_http_upload_callback = None

#
# This progress code is based on KodakLoader by Jason Hildebrand
# <jason@opensky.ca>. See http://www.opensky.ca/~jdhildeb/software/kodakloader/
# for details.
class HTTPProgressConnection(httplib.HTTPConnection):
    '''Implement a HTTPConnection with an optional callback function for
    upload progress.'''

    def send(self, data):
        global _http_upload_callback

        # if callback has not been set, call the old method
        if not _http_upload_callback:
            httplib.HTTPConnection.send(self, data)
            return

        sent = 0
        total = len(data)
        chunksize = 1024
        while sent < total:
            _http_upload_callback(sent, total)
            t1 = time.time()
            httplib.HTTPConnection.send(self, data[sent:sent+chunksize])
            sent += chunksize
            t2 = time.time()

            # adjust chunksize so that it takes between .5 and 2 
            # seconds to send a chunk
            if t2 - t1 < .5:
                chunksize *= 2
            elif t2 - t1 > 2:
                chunksize /= 2

class HTTPSProgressConnection(httplib.HTTPConnection):
    '''Implement a HTTPConnection with an optional callback function for
    upload progress.'''

    def send(self, data):
        global _http_upload_callback

        # if callback has not been set, call the old method
        if not _http_upload_callback:
            httplib.HTTPSConnection.send(self, data)
            return

        sent = 0
        total = len(data)
        chunksize = 1024
        while sent < total:
            _http_upload_callback(sent, total)
            t1 = time.time()
            httplib.HTTPSConnection.send(self, data[sent:sent+chunksize])
            sent += chunksize
            t2 = time.time()

            # adjust chunksize so that it takes between .5 and 2 
            # seconds to send a chunk
            if t2 - t1 < .5:
                chunksize *= 2
            elif t2 - t1 > 2:
                chunksize /= 2

class HTTPProgressHandler(urllib2.HTTPHandler):

    def __init__(self, progress_callback = None, debuglevel = 0):
        self._debuglevel = debuglevel
        global _http_upload_callback
        _https_upload_callback = progress_callback

    def http_open(self, req):
        return self.do_open(HTTPProgressConnection, req)

    def https_open(self, req):
        return self.do_open(HTTPSProgressConnection, req)
