#
# Copyright 2008 SUSE Linux Products GmbH
# Author: Jan Blunck <jblunck@suse.de>
#
# This library is free software;  you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
'''HTTPSConnection helper that verifies the server side certificate

This is necessay if you want to ensure that you are talking to the correct
server.

Example:
  see unittest at the end of this file
'''

import urllib2, httplib, socket
from M2Crypto import SSL
import os

class HTTPSValidateCertificateConnection(httplib.HTTPConnection):
    "This class allows communication via SSL."

    default_port = httplib.HTTPS_PORT
    cert_location = '/etc/ssl/certs/'

    def __init__(self, host, port=None, strict=None,
                 timeout=socket._GLOBAL_DEFAULT_TIMEOUT):
        httplib.HTTPConnection.__init__(self, host, port, strict, timeout)

    def connect(self):
        "Connect to a host on a given (SSL) port."

        # Setup SSL context to demand a certificate
        ctx = SSL.Context('sslv23')
        ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 10)
        ctx.load_verify_locations(HTTPSValidateCertificateConnection.cert_location)

        # Create real socket
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        ssl = SSL.Connection(ctx, sock)
        self.sock = ssl
        self.sock.connect((self.host, self.port))
        self.connected = True

    def set_cert_location(cls, value):
        cls.cert_location = value
    set_cert_location = classmethod(set_cert_location)

class HTTPSValidateCertificateHandler(urllib2.HTTPSHandler):
    def https_open(self, req):
        return self.do_open(HTTPSValidateCertificateConnection, req)

###################################################################

if __name__=="__main__":
    import unittest

    class __HTTPSValidateCertificateConnectionTest(unittest.TestCase):
        def test_connect(self):
            HTTPSValidateCertificateConnection.set_cert_location('/usr/share/apport/CA.cert')
            opener = urllib2.build_opener(HTTPSValidateCertificateHandler)
            try:
                answer = opener.open('https://imap.suse.de').read()
            except SSL.Error:
                self.fail()

    unittest.main()
